rem $DBA/resize.sql
rem
rem Creates a SQL script that can be used to resize a table.
rem
rem Restrictions:
rem   1) This script has been tested for Oracle7.
rem   2) It assumes that the table name is no more than 26 characters,
rem	 otherwise, you'll have to change the "XYZ_" prefix that is prepended
rem	 to the temporary table that is created.
rem   3) It assumes that there are no foreign key constraints on the table.
rem
rem Last Change 10/09/98 by Brian Lomasky to include temp table creation within
rem					this script.  Fix parameter numbers.
rem					Document restrictions.
rem Last Change 06/09/98 by Brian Lomasky to handle maxextents unlimited.
rem Last Change 09/30/97 by Brian Lomasky to add support for schema name.
rem Last Change 04/10/97 by Brian Lomasky to add bitmap indexes.
rem Last Change 06/26/96 by Brian Lomasky

set termout off
set heading off
set feedback off
set pagesize 0
set verify off
set newpage 0
col ext noprint new_value ext
select extents ext from sys.dba_segments
	where owner = '&&1' and segment_name = '&&2' and
	segment_type = 'TABLE';

rem ---- Create temp table containing index drop and recreation statements ----
drop table ddl_temp_indexes;
set termout on
create table ddl_temp_indexes(
	index_name	varchar2(30),
	index_owner	varchar2(30),
	table_name	varchar2(30),
	table_owner	varchar2(30),
	uniqueness	varchar2(6),
	tablespace_name	varchar2(30),
	ini_trans	number,
	max_trans	number,
	initial_extent	number,
	next_extent	number,
	min_extents	number,
	max_extents	number,
	pct_increase	number,
	pct_free	number,
	colnames1	varchar2(248),
	colnames2	varchar2(248));
declare
    comma            varchar2(1);
    ind_name         varchar2(30);
    ind_owner        varchar2(30);
    tab_name         varchar2(30);
    tab_owner        varchar2(30);
    uniq             varchar2(9);
    tbs_name         varchar2(30);
    ini_tr           number;
    max_tr           number;
    init_ex          number;
    next_ex          number;
    min_ex           number;
    max_ex           number;
    pct_inc          number;
    pct_fr           number;
    colnm1           varchar2(248);
    colnm2           varchar2(248);

    cursor ind is
           select index_name, owner, table_name, table_owner, uniqueness,
                  tablespace_name, ini_trans, max_trans, initial_extent,
                  next_extent, min_extents, max_extents, pct_increase, pct_free
           from sys.dba_indexes
           where owner = '&&1' and table_name = '&&2';
begin
     open ind;
  loop
     fetch ind into ind_name, ind_owner, tab_name, tab_owner,
                    uniq, tbs_name, ini_tr, max_tr, init_ex, next_ex,
                    min_ex, max_ex, pct_inc, pct_fr;
     colnm1 := '';
     colnm2 := '';
     comma := '';
     exit when ind%notfound;
  declare
    colname   varchar2(30);
    colpos    number;
    cursor ind_col is
         select column_name, column_position
         from sys.dba_ind_columns
         where index_name = ind_name and index_owner = ind_owner
         order by column_position;

  begin
     open ind_col;
     Loop
         fetch ind_col into colname, colpos;
         exit when ind_col%notfound;
         if colpos < 9
         then
              colnm1 := colnm1||comma||colname;
              comma := ',';
         else
              colnm2 := colnm2||comma||colname;
         end if;
     end loop;
     colnm2 := colnm2||')';
     close ind_col;
     insert into ddl_temp_indexes values (ind_name, ind_owner, tab_name,
	tab_owner, uniq, tbs_name, ini_tr, max_tr, init_ex, next_ex, min_ex,
	max_ex, pct_inc, pct_fr, colnm1, colnm2);
     commit;
  end;
  end loop;
  close ind;
end;
/

set termout off
spool resize.tmp
column seq format 99 noprint
select 2 seq,
	'create table '||OWNER||'.XYZ_&&2 tablespace '
	from sys.dba_tables where owner = '&&1' and table_name = '&&2'
union
select 3 seq,
	'  '||TABLESPACE_NAME||' storage (initial '||&ext * initial_extent||
	' next '||greatest(next_extent, floor((&ext * initial_extent) / 5))||
	' pctincrease 0)'
	from sys.dba_tables where owner = '&&1' and table_name = '&&2'
union
select 4 seq,
	'  as select * from '||OWNER||'.&&2;'
	from sys.dba_tables where owner = '&&1' and table_name = '&&2'
union
select 5 seq,
	'drop index '||index_owner||'.'||index_name||';'
	from ddl_temp_indexes
	where uniqueness <> 'UNIQUE'
union
select 6 seq,
	'drop table '||OWNER||'.&&2;'
	from sys.dba_tables where owner = '&&1' and table_name = '&&2'
union
select 7 seq,
	'rename '||OWNER||'.XYZ_&&2 to '||OWNER||'.&&2;'
	from sys.dba_tables where owner = '&&1' and table_name = '&&2'
union
select 8 seq,
	'create '|| decode(uniqueness,'UNIQUE','unique','BITMAP','bitmap',null)
		||' index '||index_owner||'.'||index_name ||' on '||chr(10)||
		chr(9)||
		lower(table_owner)||'.'||table_name||'('||chr(10)||chr(9)||
		colnames1||chr(10)||chr(9)||colnames2||chr(10)||chr(9)||
		'tablespace '||tablespace_name||chr(10)||chr(9)||
		'storage (initial '||initial_extent||' next '||next_extent||
		' pctincrease '||pct_increase||chr(10)||chr(9)||
		' minextents '||min_extents||' maxextents '||
		decode(max_extents, 2147483645, 'unlimited',
		substr(to_char(max_extents, '999999999'), 2))||')'||
		chr(10)||chr(9)||
		'pctfree '||pct_free||' initrans '||ini_trans||' maxtrans '||
		max_trans||';'
	from ddl_temp_indexes
order by 1;
spool off
drop table ddl_temp_indexes;
set termout on
exit
